import { getAround } from "./overpass";
import { Position, WithId, WithTags, ResponseItem } from "./types";

export type Bench = Position &
  WithTags &
  WithId & {
    certainty: number;
    type: BenchType;
  };
const query = `
[out:json][timeout:25];
(
nwr["amenity"="bench"]({{bbox}});
nwr["leisure"="picnic_table"]({{bbox}});
nwr["public_transport"="platform"]["shelter"!="no"]["bench"!="no"]({{bbox}});
nwr["public_transport"="platform"][!"shelter"]["bench"!="no"]({{bbox}});
nwr["public_transport"="platform"]["shelter"!="no"][!"bench"]({{bbox}});
nwr["public_transport"="platform"][!"shelter"][!"bench"]({{bbox}});
nwr["bench"="yes"]({{bbox}});
);
out geom;
`;

function getCertainty(element: ResponseItem): number {
  if (element.tags.leisure === "picnic_table") return 1;
  if (element.tags.amenity === "bench") return 1;
  if (element.tags.bench === "yes") return 1;
  if (element.tags.shelter === "yes") return 0.5;
  return 0;
}
type BenchType =
  | "bench"
  | "picnic_table"
  | "shelter"
  | "unknown"
  | "transit_stop";
function getType(element: ResponseItem): BenchType {
  if (element.tags.leisure === "picnic_table") return "picnic_table";
  if (element.tags.amenity === "bench") return "bench";

  if (element.tags.shelter === "yes") return "shelter";
  if (
    element.tags.public_transport === "platform" ||
    element.tags.shelter_type === "public_transport"
  )
    return "transit_stop";
  return "unknown";
}
function mapResults(results: ResponseItem[]): Bench[] {
  return results
    .map((element) => {
      let center: Position;
      if (element.type === "way") {
        center = {
          lat: (element.bounds.minlat + element.bounds.maxlat) / 2,
          lon: (element.bounds.minlon + element.bounds.maxlon) / 2,
        };
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
      } else if (element.type === "node") {
        center = {
          lat: element.lat,
          lon: element.lon,
        };
      } else {
        console.warn("Unknown element type", element);
        return;
      }
      return {
        id: element.id,
        ...center,
        certainty: getCertainty(element),
        type: getType(element),
        tags: element.tags,
      };
    })
    .filter((item) => !!item);
}

export async function getBenches({
  lat,
  lon,
  radius,
}: {
  lat: number;
  lon: number;
  radius: number;
}): Promise<Bench[]> {
  return mapResults(
    await getAround({
      lat,
      lon,
      radius,
      query,
    }),
  );
}
