import { distance, getBounds } from "./helpers";
import { Bounds, Position, ResponseItem } from "./types";
import { config } from "./config";

type Response = {
  version: number;
  generator: string;
  osm3s: {
    timestamp_osm_base: string;
    timestamp_areas_base?: string;
    copyright: string;
  };
  elements: ResponseItem[];
};

const boundsKeys = ["minlat", "minlon", "maxlat", "maxlon"] as (keyof Bounds)[];

export async function sendRequest({
  query,
  bounds,
}: {
  query: string;
  bounds?: Bounds;
}): Promise<Response> {
  let parsedQuery = query;
  if (bounds) {
    parsedQuery = query.replaceAll(
      "{{bbox}}",
      boundsKeys.map((key) => bounds[key]).join(","),
    );
  }
  const [response] = await Promise.all([
    fetch(`${config.overpassUrl}/api/interpreter`, {
      method: "POST",
      headers: {
        "Content-Type": "application/x-www-form-urlencoded",
        "User-Agent": config.userAgent,
      },
      body: new URLSearchParams({
        data: parsedQuery,
      }),
    }),
  ]);
  return (await response.json()) as Response;
}

export async function getAround({
  lat,
  lon,
  radius,
  query,
}: {
  lat: number;
  lon: number;
  radius: number;
  query: string;
}): Promise<ResponseItem[]> {
  const center = { lat, lon };
  const bounds = getBounds(center, radius);
  const { elements } = await sendRequest({
    query,
    bounds,
  });
  //Check if at least part of the element is within the bounds
  return elements.filter((element) => {
    let positionsToCheck: Position[];
    if (element.type === "way") {
      positionsToCheck = [
        { lat: element.bounds.minlat, lon: element.bounds.minlon },
        { lat: element.bounds.minlat, lon: element.bounds.maxlon },
        { lat: element.bounds.maxlat, lon: element.bounds.maxlon },
        { lat: element.bounds.maxlat, lon: element.bounds.minlon },
      ];
      // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition -- TODO: fix with relations.
    } else if (element.type === "node") {
      positionsToCheck = [{ lat: element.lat, lon: element.lon }];
    } else {
      console.warn("Unknown element type", element);
      return true;
    }
    return positionsToCheck.some(
      (corner) => distance(corner, { lat, lon }) <= radius,
    );
  });
}
